/*********************************************************************************
Module description:
  This file is used to finish USB-related functions,  
  Implement all necessary USB event response
  Send/Read data buffer for GP2021 to correspondent endpoints

Author : Yu Lu ,
         luyu1974@gmail.com

*********************************************************************************/


#include "reg420.h"
#include "usbn9603.h"
#include "global.h"
#include "gp2021.h"
#include "gbl_macro.h"
#include "usb.h" 

extern unsigned char write_buf_leng, write_buf_idx, frm_count,frm_count1;
extern unsigned int data usb_snd_idx,buf_idx;

extern void puthex_char(char ch);
extern void puthex_int(unsigned int a);
extern void put_char(char ch);

/*************************************
sbit GPS_ALE =  P2^0;
sbit GPS_WR  =  P2^1;
sbit GPS_CS  =  P2^2;
sbit GPS_RD  =  P2^3;
sbit GPS_OE  =  P2^4;
**************************************/
extern void DELAY_8US();
extern void DELAY_50US();
extern void DELAY_1MS();
extern void read_gps(char add, unsigned int* datadd);
extern void write_gps(char add, char lo, char hi);
extern void write_gps_noint(char add, char lo, char hi);

extern char read_usb(unsigned int usbadr);
extern void write_usb(unsigned int usbadr, char dta);
extern void bunch_write_usb( unsigned int usbadr, unsigned int datadd, unsigned char count);
extern void bunch_read_usb( unsigned int usbadr, unsigned int datadd, unsigned char count);


/*
 *
 *	the buffer is :
     rsnc = read_usb(RXD1);
	 rcmd = read_usb(RXD1);
	 rdta = read_usb(RXD1);
	 radh = read_usb(RXD1);
	 radl = read_usb(RXD1);
	 rcks = read_usb(RXD1);
 */
void do_cmd()
{
	unsigned char tmpch;  // gps_part indicates which part of GP2021 regs is get this time
	unsigned char *radr;
	unsigned char idata tmp_buf[64];
	

	radr = (unsigned char *)radl;

switch (rcmd)
{
 case 'g':                      // read the vale of USBN9603's registers
 	tmpch = read_usb((unsigned int)(0x7E00+radl));

	FLUSHTX1;
	write_usb(TXD1,tmpch);
	//TXEN1_PID_NO_TGL;
	TXEN1_PID_NO_TGL;

	break;
 case 'r':

#if DEBG == 1
    puthex_char(radr);
	puthex_char(radl);
#endif 
	FLUSHTX1;
 	write_usb(TXD1,*radr);     // read the value of C51's registers
	TXEN1_PID_NO_TGL;
	break;
 case 'w':                     // write the value of C51's registers
 	*radr = rdta;
	FLUSHTX1;
	write_usb(TXD1,*radr);
	TXEN1_PID_NO_TGL;
	break;

 case 'q':                     // read the value of GP2021's registers
	read_gps(rdta, (unsigned int*)gps_data);
	FLUSHTX1;
	write_usb(TXD1,gps_data[0]);
	write_usb(TXD1,gps_data[1]);
	TXEN1_PID_NO_TGL;
 	break;

 case 't':                    // write the value of GP2021's registers

/*       just for debugging
	puthex_char(rdta);
    puthex_char(radh);
	puthex_char(radl);
*/
	write_gps(rdta,radl,radh );
	
	break;
 case 'a':                    // send all the GP2021's regs 

    sts_send_gp2021_all = 1;  // set flag to indicate we need to transfer gp2021's reg

 	break;

 case 'b':
 	get_usbn9603(tmp_buf);
	send_all_usbn9603_TXD1(tmp_buf);
	break;
 case 'c':                   // control perfomance of gp2021
 	process_gp2021(rdta);
	

 default:
//	write_usb(TXD1,'?');
;
}

}

/*********************************************************************
 delay  to reset usbn9603
 NOTE: 
 P2 = 0x7E,  chip select usbn9603
 P2 = 0xBE,  reset usbn9603
 P2 = 0xFE,  idle state
*********************************************************************/
void rst_usb(void)
{

 P2 = 0xBE;  // reset usbn9603
 DELAY_50US();
 DELAY_1MS();
 DELAY_1MS();
 DELAY_1MS();
 P2 = 0xFE;
 DELAY_1MS();
 DELAY_1MS();
}
/**********************************************************************/
/* This subroutine initializes the 9602.                              */
/**********************************************************************/
void init_usb(void)
{
 /*toss out any previous state ***********************************/
 status = 0;
 sts_getdesc = 0;
 usb_cfg = 0;
 write_buf_leng = 0;
 write_buf_idx = 0;
 /*give a software reset, then set ints to active low push pull */
 write_usb(MCNTRL,SRST);
 while( read_usb(MCNTRL) & SRST);

 write_usb(MCNTRL,INT_L_P + VGE );  

 write_usb(MAMSK, 0);   // disable all ints                                 
	
 /*initialize the clock generator ********************************/
 /* prior to this point, the clock output will be 4 Mhz.  After, */    
 /* it will be (48 MHz/CLKDIV)                                   */ 
 write_usb(CCONF,CLKDIV-1); 

 /*set up interrupt masks ****************************************/
 write_usb(NAKMSK,NAK_O0);                          /*NAK evnts*/  
 write_usb(TXMSK,TXFIFO0+TXFIFO1+TXFIFO2+TXFIFO3);          /*TX events*/
 write_usb(RXMSK,RXFIFO0+RXFIFO1+RXFIFO2+RXFIFO3);          /*RX events*/
 write_usb(ALTMSK,SD3+RESET_A);                     /*ALT evnts*/
 /*this is modified in the */
 /*suspend-resume routines */
 /*so if any change is made*/
 /*here it needs to be ref-*/
 /*lected there too.       */
 write_usb(MAMSK,(INTR_E+RX_EV+NAK+TX_EV+ALT));
 write_usb(DMAMASK, 0);

 write_usb(0x7E1f, 0x40);

 /*set default address, enable EP0 only **************************/
 write_usb(FAR,AD_EN+0);
 write_usb(EPC0, 0x00); 
 /*enable the receiver and go operational ************************/  
 FLUSHTX0;                           /*flush TX0 and disable   */  
  
 write_usb(RXC0,RX_EN);              /*enable the receiver     */

 write_usb(NFSR,OPR_ST);             /*go operational          */   
 write_usb(MCNTRL,INT_L_P+NAT+VGE);      /*set NODE ATTACH         */
}
/**********************************************************************/
/* This subroutine handles USB 'alternate' events.                    */
/**********************************************************************/
void usb_alt(void)
  {
    evnt = read_usb(ALTEV);             /*check the events        */   
	if(evnt & RESET_A)                  /*reset event             */
	  {
	    write_usb(NFSR,RST_ST);         /*enter reset state       */   
	    write_usb(FAR,AD_EN+0);         /*set default address     */
	    write_usb(EPC0,0x00);          /*enable EP0 only         */
	    FLUSHTX0;                       /*flush TX0 and disable   */   
	    write_usb(RXC0,RX_EN);          /*enable the receiver     */
		write_usb(ALTMSK,SD3+RESET_A);
	    write_usb(NFSR,OPR_ST);         /*go operational          */ 
	  }

	else if(evnt & (SD3 | SD5) )                 /*suspend event           */ 
	  {
	    write_usb(ALTMSK,RESUME_A + RESET_A);  /*adjust interrupts  */   
	    write_usb(NFSR,SUS_ST);         /*enter suspend state     */  
	  }

	else if(evnt & RESUME_A)            /*resume  event           */ 
	  {
	    write_usb(ALTMSK,SD3+RESET_A);       /*adjust interrupts  */  
	    write_usb(NFSR,OPR_ST);         /*go operational          */ 
      //  write_usb(RXC0,RX_EN); 
	  }
	else                                /*spurious alt. event!    */
	  {
	  }
	
  }
/**********************************************************************/
/* The CLEAR_FEATURE request is done here                             */  
/**********************************************************************/
void clrfeature(void)
  {
	switch(usb_buf[0]&0x03)            /*find request target     */
	  {
	    case 0:                         /*DEVICE                  */  
	      break;

	    case 1:                         /*INTERFACE               */  
	      break;

	    case 2:                         /*ENDPOINT                */  
	      switch (usb_buf[3])           /*find specific endpoint  */
		{
		  case 0:                                                 
		    stall_ep0 = 0;
		    break;
		  case 1:                                                 
		    stall_ep1 = 0;
		    break;
		  case 2:                                                 
		    stall_ep2 = 0;
		    break;
		  case 3:                                                 
		    stall_ep3 = 0;
		    break;
		  case 4:                                                 
		    stall_ep4 = 0;
		    break;
		  case 5:                                                 
		    stall_ep5 = 0;
		    break;
		  case 6:                                                 
		    stall_ep6 = 0;
		    break;
		  default:
		    break;
		}
	      break;

	    default:                        /*UNDEFINED               */ 
	      break;
	  }
  }
/**********************************************************************/
/* The GET_DESCRIPTOR request is done here                            */  
/**********************************************************************/
void getdescriptor(void)
  {
	sts_getdesc=1;           /*enter get_descr mode    */
	desc_typ = usb_buf[3];      /*store the type requested*/
	string_idx = usb_buf[2];    /*store the index */
	if(desc_typ==DEVICE) 
	 desc_sze = DEV_DESC_SIZE;
	else if(desc_typ==CONFIGURATION) 
	 desc_sze = CFG_DESC_SIZE;
	else if(desc_typ==XSTRING )
		desc_sze = string_sze[ string_idx ];

      /*adjust size, if the host has asked for less than we   */ 
      /*want to send.  Note that we only check the low order  */ 
      /*byte of the wlength field.  If we ever need to send   */ 
      /*back descriptors longer than 256 bytes, we'll need to */ 
      /*revisit this.                                         */ 
	if (desc_sze > usb_buf[6]) 
	 desc_sze = usb_buf[6];    

      /*send the first data chunk back                        */
	for(desc_t_idx=0; ((desc_t_idx<8)&&(desc_t_idx<desc_sze)); desc_t_idx++) get_desc();  
  }

/**********************************************************************/
/* This subroutine loads a byte from a descriptor into endpoint 0     */
/* Fifo.                                                              */  
/**********************************************************************/
void get_desc(void)
  {
    char desc_dta;                        
    
       /*select the appropriate descriptor.  compiler limits forced*/  
       /*the code to be written in this (admittedly) akward way.   */  
	if(desc_typ==DEVICE) 
	  {
	    desc_dta=DEV_DESC[desc_t_idx];    
	  }
	else if(desc_typ==CONFIGURATION) 
	  desc_dta=CFG_DESC[desc_t_idx]; 
	else if(desc_typ==XSTRING)
	  desc_dta=((char*)(string_descs[string_idx]))[desc_t_idx];


	write_usb(TXD0,desc_dta);           /*send data to the FIFO   */

#if DEBG == 1
    puthex_char(desc_t_idx); 
    puthex_char(desc_dta);
#endif
  }



/**********************************************************************/
/* The GET_STATUS request is done here                                */  
/**********************************************************************/
void getstatus(void)
  {
	switch (usb_buf[0]&0x03)            /*find request target     */
	  {
	    case 0:                         /*DEVICE                  */  
	      write_usb(TXD0,0);            /*first  byte is reserved */  
	      break;

	    case 1:                         /*INTERFACE               */  
	      write_usb(TXD0,0);            /*first  byte is reserved */   
	      break;

	    case 2:                         /*ENDPOINT                */  
	      switch (usb_buf[3])           /*find specific endpoint  */
		{
		  EPSTATUS(0); EPSTATUS(1); EPSTATUS(2); EPSTATUS(3);  
		  EPSTATUS(4); EPSTATUS(5); EPSTATUS(6);  
		  default:
		    break;
		}
	      break;

	    default:                        /*UNDEFINED               */   
	      break;
	  }

	write_usb(TXD0,0);                  /*second byte is reserved */   
  }
/**********************************************************************/
/* The SET_CONFIGURATION request is done here                         */  
/**********************************************************************/
void setconfiguration(void)
  {
	usb_cfg = usb_buf[2];               /*set the configuration # */  
	if (usb_buf[2]!=0)                  /*set the configuration   */   
	  {
	    dtapid = 0;                     /*FIRST PID is DATA0      */   
	    stalled = 0;                     /*nothing stalled         */
		
	    FLUSHTX1;                       /*flush  TX1 and disable  */   
	    write_usb(EPC1,EP_EN+01);       /*enable EP1 at adr 1     */

	    FLUSHRX1;                       /*flush  RX1 and disable  */ 
	    write_usb(EPC2,EP_EN+02);       /*enable EP2 at adr 2     */   
	    write_usb(RXC1,RX_EN);          /*enable RX1              */     
		
	    FLUSHTX3;                       /*flush  TX3 and disable  */ 
		                                /* double buffer here     */
		write_usb(EPC3,EP_EN+05);       /*enable EP3 at adr 5     */   
	    write_usb(EPC5,EP_EN+05);       /*enable EP5 at adr 5     */   

	    FLUSHRX3;                       /*flush  RX3 and disable  */ 
										/* double buffer here     */
		write_usb(EPC4,EP_EN+06);       /*enable EP4 at adr 6     */  
 	    write_usb(RXC2,RX_EN);          /*enable RX2              */ 
	    write_usb(EPC6,EP_EN+06);       /*enable EP6 at adr 6     */   
	    write_usb(RXC3,RX_EN);          /*enable RX3              */     
	  }
	else                                /*unconfigure the device  */
	  {
	    write_usb(EPC1,0);              /*disable EP1             */
	    write_usb(EPC2,0);              /*disable EP2             */   
	    write_usb(EPC3,0);              /*disable EP3             */
	    write_usb(EPC4,0);              /*disable EP4             */
	    write_usb(EPC5,0);              /*disable EP5             */   
	    write_usb(EPC6,0);              /*disable EP6             */   
	  }
  }
/**********************************************************************/
/* The SET_FEATURE request is done here                               */  
/**********************************************************************/
void setfeature(void)
  {
	switch (usb_buf[0]&0x03)            /*find request target     */
	  {
	    case 0:                         /*DEVICE                  */  
	      break;

	    case 1:                         /*INTERFACE               */  
	      break;

	    case 2:                         /*ENDPOINT                */  
	      switch (usb_buf[3])           /*find specific endpoint  */
		{
		  case 0:                                                 
		    stall_ep0 = 1;
		    break;
		  case 1:                                                 
		    stall_ep1 = 1;
		    break;
		  case 2:                                                 
		    stall_ep2 = 1;
		    break;
		  case 3:                                                 
		    stall_ep3 = 1;
		    break;
		  case 4:                                                 
		    stall_ep4 = 1;
		    break;
		  case 5:                                                 
		    stall_ep5 = 1;
		    break;
		  case 6:                                                 
		    stall_ep6 = 1;
		    break;
		  default:
		    break;
		}
	      break;

	    default:                        /*UNDEFINED               */   
	      break;
	  }
  }


/**************************************************************************
 This subroutine is used to get and send GP2021 regs in one transaction
 The values of regs are stored in gp2021_regbuf0
 This function just init the transaction , the rest are done in int service
 This transaction is finished by TXD1
**************************************************************************/
void send_all_gp2021_TXD1(void )
{
 sts_send_gp2021_TXD1 = 1;
 FLUSHTX1;
 bunch_write_usb(TXD1, (unsigned int)gp2021_reg_buf, 64);
 gp2021_TXD1_idx = 64; 
 TXEN1_PID_NO_TGL;
}

/**************************************************************
 This subroutine is used to get and send GP2021 regs in one transaction
 The values of regs are stored in gp2021_regbuf0
 This function just init the transaction , the rest are done in int service
 This transaction is finished by TXD3 & TXD2, double buffers used
**************************************************************/
void send_all_gp2021_TXD3( void )
{
	sts_send_gp2021_TXD23 = 1;
	sts_send_first_end = 1;

	FLUSHTX2;               // fill TXD2 fifo first
	bunch_write_usb(TXD2, (unsigned int)gp2021_reg_buf + usb_snd_idx, 64);

	FLUSHTX3;               // fill TXD3 fifl then
	bunch_write_usb(TXD3, (unsigned int)(gp2021_reg_buf + usb_snd_idx + 64), 64);
	gp2021_TXD23_idx = 128; 
    
	write_usb(EPC3,EP_EN+05);       /*enable EP3 at adr 5     */   
    write_usb(EPC5,EP_EN+05);       /*enable EP5 at adr 5     */ 

	write_usb(TXC2,TX_LAST+TX_EN);           // Begin TXD2 transmit, DATA0
	write_usb(TXC3,TX_TOGL+TX_LAST+TX_EN);   // Begin TXD3 transmit, DATA1

}

/**************************************************************************
 This subroutine is to get and send all the regs of USBN9603 
 For debugging only
**************************************************************************/
void get_usbn9603(unsigned char* tmp_b)
{
 unsigned char i;
 for(i =0; i<64; i++)
 tmp_b[i] = read_usb((unsigned int)(0x7E00+i));
}

void send_all_usbn9603_TXD1(unsigned char* tmp_b)
{
 unsigned char i;
 FLUSHTX1;
 for(i =0; i<63; i++)
 write_usb(TXD1, tmp_b[i]);
 TXEN1_PID_NO_TGL;
}


/**********************************************************************/
/* This subroutine handles RX events for FIFO0 (endpoint 0)           */  
/**********************************************************************/
void rx_0(void)
{
 rxstat=read_usb(RXS0);              /*get receiver status     */    

#if DEBG ==1
 put_char('R');
 puthex_char(rxstat);
#endif
 
 /*is this a setup packet? ***************************************/  
 if(rxstat & SETUP_R)                   
 { 
  /*read data payload into buffer then flush/disble the RX ****/
  for(desc_idx=0; desc_idx<8; desc_idx++)
  {
   usb_buf[desc_idx] = read_usb(RXD0);
   //puthex_char(usb_buf[desc_idx]);
  }

  FLUSHRX0;                     /*make sure the RX is off */
  FLUSHTX0;                     /*make sure the TX is off */ 

  if ((usb_buf[0]&0x60)==0x00)    /*if a standard request   */
   switch (usb_buf[1])           /*find request target     */
   {
    case CLEAR_FEATURE:                  
     clrfeature(); 
     break;

    case GET_CONFIGURATION:                   
     write_usb(TXD0,usb_cfg);/*load the config value   */ 
     break;
 
    case GET_DESCRIPTOR: 
     getdescriptor();                   
     break;
 
    case GET_STATUS: 
     getstatus();   
	 break;

    case SET_ADDRESS:
		  /*set and enable new address for endpoint 0, but set*/
		  /*DEF too, so new address doesn't take effect until */
		  /*the handshake completes                           */
     write_usb(EPC0,DEF);   
	 write_usb(FAR,usb_buf[2] | AD_EN);      
	 break;

	case SET_CONFIGURATION:
	 setconfiguration(); 
	 break;

	case SET_FEATURE: 
	 setfeature();  
	 break;

	default:                  /*unsupported standard req*/  
	 break;
   }
   else                            /*if a non-standard req.  */   
   {
   }

	  /*the following is done for all setup packets.  Note that if*/  
	  /*no data was stuffed into the FIFO, the result of the fol- */  
	  /*lowing will be a zero-length response.                    */ 
   write_usb(TXC0,TX_TOGL+TX_EN);  /*enable the TX (DATA1) */  
   dta_pid0=0;               /*store NEXT PID state  */
  }

   /*if not a setup packet, it must be an OUT packet ***************/ 
  else   
  {
   if (sts_getdesc)             /*get_descr status stage? */
   {
    /*test for errors (zero length, correct PID)            */
    if ((rxstat& 0x5F)!=0x10)   /*length error??          */
    {
    }
					    
    sts_getdesc=0;           /*exit get_descr mode     */
	FLUSHTX0;                   /*flush TX0 and disable   */   
   }
  write_usb(RXC0,RX_EN);          /*re-enable the receiver  */   
 }
 //test_b= 0;
 /*we do this stuff for all rx_0 events **************************/  
}

/**********************************************************************/
/* This subroutine handles RX events for FIFO1 (endpoint 2)           */  
/**********************************************************************/
void rx_1(void)
{
	rxstat = read_usb(RXS1);

	if( rxstat & SETUP_R)
	{}
	else if(rxstat & RX_ERR)
	{}
	else
	{
	 sts_usbcmd = 1;
	 rsnc = read_usb(RXD1);
	 rcmd = read_usb(RXD1);
	 rdta = read_usb(RXD1);
	 radh = read_usb(RXD1);
	 radl = read_usb(RXD1);
	 rcks = read_usb(RXD1);
    
	 if(rsnc == SYNCBYT)
	 if(rcks = rsnc + rcmd + rdta + radh +radl)
	 {
      do_cmd();
	 }
	sts_usbcmd = 0;
	}

   if(!sts_send_gp2021_TXD1)  /* if we are sending all the gp2021 regs, we need wait for other transfers*/
   {
    FLUSHRX1;
    write_usb(RXC1, RX_EN);
   }
}
/**********************************************************************/
/* This subroutine handles RX events for FIFO2 (endpoint 4)           */  
/**********************************************************************/
void rx_2(void)
{
 rxstat = read_usb(RXS2);
 if( (rxstat & RX_ERR) || !(rxstat & RX_LAST) || (rxstat & RX_TOGL) ) /* some error happened */
 { 
  FLUSHRX2;
  write_usb(RXC2, RX_EN);
 }
 else
 {
  bunch_read_usb(RXD2, (unsigned int)(gp2021_reg_write), 64);  
  FLUSHRX2;
  write_usb(EPC4,0);   /* disable RX2 temporarily */
 }
}

/**********************************************************************/
/* This subroutine handles RX events for FIFO3 (endpoint 6)           */  
/**********************************************************************/
void rx_3(void)
{
 rxstat = read_usb(RXS3);
 if( (rxstat & RX_ERR) || !(rxstat & RX_LAST) || !(rxstat & RX_TOGL) ) /* some error happened */
 {
    FLUSHRX3;
    write_usb(RXC3, RX_EN);
 }
 else
 {
  bunch_read_usb(RXD3, (unsigned int)(gp2021_reg_write+64), 61);  /* whole packet length is 125 */
  FLUSHRX3;
  write_usb(RXC3, RX_EN);
  write_usb(EPC4,EP_EN+06);       /*enable EP4 at adr 6     */  
  write_usb(RXC2,RX_EN);          /*enable RX2              */ 
  get_data_ready = 1;             /* set the data ready flag*/

  sts_send_gp2021_all = gp2021_reg_write[0] & 0x01;        /* set flag to indicate we need to transfer gp2021's reg*/
  gp2021_reg_buf[324 + buf_idx ] = gp2021_reg_write[0];  // dummy[0]
  gp2021_reg_buf[325 + buf_idx ] = frm_count1;  // dummy[1]
  gp2021_reg_buf[326 + buf_idx ] = frm_count;   // dummy[2]
                                  /* first byte is flag to indicate this transaction requires return or not */

 }
}
/*********old code ***************************************************/
/*
{
	rxstat = read_usb(RXS3);
    puthex_char(rxstat);

	if( rxstat & SETUP_R)
	{}
	else if(rxstat & RX_ERR)
	{}
	else
	{
		if( write_buf_leng > 0)		 // indicate that this package is the last one's continue;
		{
			if( write_buf_leng > write_buf_idx + 64 )
			{
				bunch_read_usb(RXD3, (unsigned int)(gp2021_reg_write+write_buf_idx), 64);
				write_buf_idx += 64;
				
			}
			else
			{
				bunch_read_usb(RXD3, (unsigned int)(gp2021_reg_write+write_buf_idx), write_buf_leng-write_buf_idx+1);
				write_buf_idx  = 0;	
				write_buf_leng = 0;    // the end of the whole transaction
				get_data_ready = 1;   // set the data ready flag;
				sts_send_gp2021_all = 1; // set flag to indicate we need to transfer gp2021's reg 
			}
		}
		else	                     // else this is a new package;
		{
			write_buf_leng = read_usb(RXD3);  // the first byte is the total length
			gp2021_reg_write[0] = write_buf_leng;

			if(write_buf_leng > 64)   // if this is only the first part of the whole transaction
			{
				bunch_read_usb(RXD3, (unsigned int)(gp2021_reg_write+1), 63);
				write_buf_idx = 64;
			}
			else   // if this package is the only package
			{
				bunch_read_usb(RXD3, (unsigned int)(gp2021_reg_write+1), write_buf_leng-1);
				write_buf_idx = 0;
				write_buf_leng = 0;
				get_data_ready = 1;   // set the data ready flag;
				sts_send_gp2021_all = 1; // set flag to indicate we need to transfer gp2021's reg 
			}		
		} // end of if( write_buf_leng > 0)
	}

    FLUSHRX3;
    write_usb(RXC3, RX_EN);
}
*/
/**********************************************************************/
/* This subroutine handles TX events for FIFO0 (endpoint 0)           */
/**********************************************************************/
void tx_0(void)
{
 unsigned char lim;

 txstat=read_usb(TXS0);              /*get transmitter status  */ 
#if DEBG ==1
 put_char('T');
 puthex_char(txstat);
 puthex_char(desc_t_idx);
#endif 

  /*if a transmission has completed successfully, check to see if */
  /*we have anything else that needs to go out, otherwise turn the*/ 
  /*receiver back on **********************************************/  

 if ((txstat & ACK_STAT) && (txstat & TX_DONE))
 {
  FLUSHTX0;                       /*flush TX0 and disable   */  
	    
  /*the desc. is sent in pieces; queue another piece if nec.  */
  if(sts_getdesc)
  {
   lim=desc_t_idx+8;             /*set new max limit       */

#if DEBG ==1
   put_char('T');
   puthex_char(desc_t_idx);
   puthex_char(lim);
#endif

   /*move the data into the FIFO   */  
   for(; ((desc_t_idx<lim)&&(desc_t_idx<desc_sze)); desc_t_idx++)
    get_desc();

#if DEBG == 1
   put_char('E');
   puthex_char(lim);
   puthex_char(desc_sze);
   puthex_char(desc_t_idx);
#endif

   if(desc_t_idx == (desc_sze -1))
    desc_t_idx = 0;

   TXEN0_PID;                  /*enable TX, choose PID   */
  }
  else
  {
   write_usb(RXC0,RX_EN);      /*re-enable the receiver  */ 
  }
 }

  /*otherwise something must have gone wrong with the previous ****/
  /*transmission, or we got here somehow we shouldn't have ********/ 
 else
 {
 }

   /*we do this stuff for all tx_0 events **************************/  
}

/**********************************************************************/
/* This subroutine handles TX events for FIFO1 (endpoint 1)           */
/**********************************************************************/
void tx_1(void)
{
 unsigned int gp2021_lim;
 unsigned char count;

 txstat = read_usb(TXS1);
 if((txstat & ACK_STAT) && (txstat & TX_DONE))
 {
  dta_pid1 = !dta_pid1;  // reverse data0 and data1
  FLUSHTX1;

  if( sts_send_gp2021_TXD1 )  /* we still need to send regs value */
  {
   gp2021_lim = gp2021_TXD1_idx +64;

   count = gp2021_lim <636 ? 64:636-gp2021_TXD1_idx;
   bunch_write_usb( TXD1, (unsigned int)(gp2021_reg_buf+gp2021_TXD1_idx), count);
   gp2021_TXD1_idx += count;


   if( gp2021_TXD1_idx == 636 )
   {
    gp2021_TXD1_idx = 0;
	sts_send_gp2021_TXD1 = 0;
   }
   TXEN1_PID_NO_TGL;
  } // end of if( sts_send_gp2021_TXD1 )
  else
  {
      FLUSHRX1;
  	  write_usb(RXC1, RX_EN);
  }
 }// end of  if((txstat & ACK_STAT) && (txstat & TX_DONE))
}

/**********************************************************************/
/* This subroutine handles TX events for FIFO2 (endpoint 3)           */
/**********************************************************************/
void tx_2(void)
{
	unsigned int data gp2021_lim ;
	unsigned char count;
    // put_timestamp();
	txstat = read_usb(TXS2);
	if((txstat & ACK_STAT) && (txstat & TX_DONE))
	{
		FLUSHTX2;
		if( sts_send_gp2021_TXD23 )  /* we still need to send regs value */
		{
//			put_char('a');
			write_usb(EPC3, 0);    // disable TXD2 temporarily

  			gp2021_lim = gp2021_TXD23_idx + 64;
          
			count = gp2021_lim <ONE_FRM_SIZE ? 64:ONE_FRM_SIZE - gp2021_TXD23_idx;
            
			bunch_write_usb( TXD2, (unsigned int)(gp2021_reg_buf + usb_snd_idx + gp2021_TXD23_idx), count);
			
			gp2021_TXD23_idx += count;

			write_usb(TXC2,TX_LAST+TX_EN);           // Begin TXD2 transmit, DATA0

			if( gp2021_TXD23_idx == ONE_FRM_SIZE )
			{
				gp2021_TXD23_idx = 0;
				sts_send_gp2021_TXD23 = 0;
			}
		}// end of if( sts_send_gp2021_TXD23 )
		else // this transaction is done
		{
//			put_char('^');
			sts_send_first_end = 0;
 			write_usb(EPC3,0); 
                                   // we need disable EPC3 because ONE_FRM_SIZE(352) bytes means 6 packets
			                       // They are : 
								   //   1: DATA0:  EPC3  
								   //   2: DATA1:  EPC5
								   //   3: DATA0:  EPC3   , gp2021_lim = 192, count = 64
								   //   4: DATA1:  EPC5   , gp2021_lim = 256, count = 64
								   //   5: DATA0:  EPC3   , gp2021_lim = 320, count = 64
								   //   6: DATA1:  EPC5   , gp2021_lim = 352, count = 32
								   
								   //  Program here means the 13th packet is done,  we still have one packet in TXD3
								   //  In order for that packet to be transmitted, we must disable EPC3 here.

			FLUSHRX2;
			write_usb(RXC2, RX_EN);
		}
	}// end of if((txstat & ACK_STAT) && (txstat & TX_DONE))
	else if(!(txstat & ACK_STAT) && (txstat & TX_DONE))
	{
		write_usb(TXC2,REF+TX_LAST+TX_EN);
	}
	else
	{
	}
}

/**********************************************************************/
/* This subroutine handles TX events for FIFO3 (endpoint 5)           */
/**********************************************************************/
void tx_3(void)
{
	unsigned int data gp2021_lim ;
	unsigned char count;
	//  put_timestamp();
	txstat = read_usb(TXS3);

	if((txstat & ACK_STAT) && (txstat & TX_DONE))
	{
		FLUSHTX3;
		if( sts_send_gp2021_TXD23 )  /* we still need to send regs value */
		{
//			put_char('b');
			write_usb(EPC3,EP_EN+05);    // enable TXD2 again

		    gp2021_lim = gp2021_TXD23_idx + 64;
   
			count = gp2021_lim <ONE_FRM_SIZE ? 64:ONE_FRM_SIZE - gp2021_TXD23_idx;

			bunch_write_usb( TXD3, (unsigned int)(gp2021_reg_buf + usb_snd_idx + gp2021_TXD23_idx), count);

			gp2021_TXD23_idx += count;

			write_usb(TXC3,TX_TOGL+TX_LAST+TX_EN);           // Begin TXD2 transmit, DATA0

			if( gp2021_TXD23_idx == ONE_FRM_SIZE )
			{
				gp2021_TXD23_idx = 0;
				sts_send_gp2021_TXD23 = 0;
			}
		} // end of if( sts_send_gp2021_TXD23 )
		else 
		{
	//		put_char('*');
			sts_send_first_end = 0;

			FLUSHRX3;
			write_usb(RXC3, RX_EN);
		} 
	} // end of if((txstat & ACK_STAT) && (txstat & TX_DONE))
	else if(!(txstat & ACK_STAT) && (txstat & TX_DONE))
	{
		write_usb(TXC3,REF+TX_TOGL+TX_LAST+TX_EN); 
	}
	else
	{
	}
}


/**********************************************************************/
/* This subroutine handles OUT NAK events for FIFO0 (endpoint 0)      */
/**********************************************************************/
void nak0(void)
{
     /*important note:  even after servicing a NAK, another NAK      */
     /*interrupt may occur if another 'OUT' or 'IN' packet comes in  */
     /*during our NAK service.                                       */
     /*if we're currently doing something that requires multiple 'IN'*/
     /*transactions, 'OUT' requests will get NAKs because the FIFO is*/ 
     /*busy with the TX data.  Since the 'OUT' here means a premature*/  
     /*end to the previous transfer, just flush the FIFO, disable the*/  
     /*transmitter, and re-enable the receiver.                      */  
 if (sts_getdesc)                   /*get_descr status stage? */   
 {
  sts_getdesc=0;                    /*exit get_descr mode     */
  FLUSHTX0;                         /*flush TX0 and disable   */  
  write_usb(RXC0,RX_EN);            /*re-enable the receiver  */   
 }

     /*we do this stuff for all nak0 events **************************/   
}

/**********************************************************************/
/* This subroutine handles OUT NAK events for FIFO1 (endpoint 2)      */
/**********************************************************************/
void nak1(void)
{
}

/**********************************************************************/
/* This subroutine handles OUT NAK events for FIFO2 (endpoint 4)      */
/**********************************************************************/
void nak2(void)
{
}

/**********************************************************************/
/* This subroutine handles OUT NAK events for FIFO3 (endpoint 6)      */
/**********************************************************************/
void nak3(void)
{
}

void usb_isr(void) interrupt 0
{
 evnt = read_usb(MAEV);              /*check the events        */

#if DEBG ==1
 puthex_char(evnt);
 put_char(' ');
#endif

  if (evnt & RX_EV) 
  {
   evnt=read_usb(RXEV);            /*check the RX events     */  

   if      (evnt&RXFIFO0) rx_0();  /*endpoint 0              */   
   else if (evnt&RXFIFO1) rx_1();  /*endpoint 2              */ 
   else if (evnt&RXFIFO2) rx_2();  /*endpoint 4              */  
   else if (evnt&RXFIFO3) rx_3();  /*endpoint 6              */  
   else                            /*some other RX event     */
   { 
   }
  }

  else if (evnt & TX_EV) 
  {
   evnt=read_usb(TXEV);            /*check the TX events     */  
 //  puthex_char(evnt);
   if      (evnt&TXFIFO0) tx_0();  /*endpoint 0              */   
   else if (evnt&TXFIFO1) tx_1();  /*endpoint 1              */ 
   else if (evnt&TXFIFO2) tx_2();  /*endpoint 3              */  
   else if (evnt&TXFIFO3) tx_3();  /*endpoint 5              */  
   else                            /*some other TX event     */
   { 
   }
  }

  else if (evnt & ALT) usb_alt();     /*alternate event?        */   
 
  else if (evnt & NAK) 
  {
   evnt=read_usb(NAKEV);           /*check the NAK events    */  
#if DEBG == 1
   put_char('N');
   puthex_char(evnt);
#endif
   
   if      (evnt & NAK_O0) nak0();   /*endpoint 0              */   
   else if (evnt & NAK_O1) nak1();   /*endpoint 2              */ 
   else if (evnt & NAK_O2) nak2();   /*endpoint 4              */  
   else if (evnt & NAK_O3) nak3();   /*endpoint 6              */  
   else                            /*some other TX event     */
   { 
   }
  }

  else                                /*spurious event!         */
  {
  }
 }


/* function to process data from host */
/* data structure:
            address ::   description 
			     0  ::   flag to transfer data or not
				 1  ::   total length of data needed to write to gp2021
			  /  3  ::   address to be written
			  |	 4  ::   low byte
			  \	 5  ::   high byte 
			  /	 .  ::   ...
			  |	 .  ::   ...
			  \	 .  ::   ...
 */

/* process data without int enabled              */
/* the reason why this function exist is that    */
/* to call this funtcion in timer0's int handler */
void process_host_data(void)
{
  unsigned char idx, add, high, low;//,i;

  /*
  for(i=0; i<gp2021_reg_write[1]+1;i++)
   gp2021_reg_buf[buf_idx+224+i] = gp2021_reg_write[i];
  for(i=(gp2021_reg_write[1]+2);i<128;i++)
   gp2021_reg_buf[buf_idx+224+i] = 0x55;
*/
 for(idx  = 0; idx < gp2021_reg_write[1]; idx++)
 {
  add = gp2021_reg_write[3*idx+2];
  low = gp2021_reg_write[3*idx+3];
  high = gp2021_reg_write[3*idx+4];
  write_gps_noint( add,  low,  high);
 }
}

/* process to control gp2021 tracking loop  */
/* commd: 0  ---  stop timer , thus stop reading gp2021
          1  ---  start timer, thus start reading gp2021
*/
void process_gp2021(char commd)
{
 switch(commd){
  case 0:  ET0 = 0; frm_count = 0; frm_count1 = 0; write_gps(0x7f, 0,0);break;
  case 1:  ET0 = 1; break;
  default: ;
  }
}

